<?php
/**
 * Functions to register client-side assets (scripts and stylesheets) for the
 * Gutenberg block.
 *
 * @package eelv-newsletter
 */

function newsletteredit_post_render( $attributes ) {
	global $eelv_newsletter;
	if(!$eelv_newsletter->currentTemplate){
		return '';
	}
	$template_id =  $eelv_newsletter->currentTemplate->ID;
	$styles = shortcode_atts($eelv_newsletter->default_item_style, get_post_meta($template_id, 'item_style',true));
	return $eelv_newsletter->single_included((object) array(
		'ID' => $attributes['post_id'],
		'post_title' => $attributes['title'],
		'post_content' => $attributes['content']
	), $styles, $styles['t_size'], isset($attributes['hasShareButtons']) ? $attributes['hasShareButtons'] : false);
}

/**
 * Registers all block assets so that they can be enqueued through Gutenberg in
 * the corresponding context.
 *
 * @see https://wordpress.org/gutenberg/handbook/blocks/writing-your-first-block-type/#enqueuing-block-scripts
 */
function newsletteredit_block_init() {
	if(!function_exists('register_block_type')){
		return;
	}
	global $eelv_newsletter;
	$dir = dirname( __DIR__ ).'/';
	$url = plugins_url( '/', __DIR__ );

	$blocks = array(
		'autocomplete',
		'post',
		'reply',
	);

	foreach($blocks as $block){

	// Reply
		$js_path = 'js/blocks/'.$block.'.js';
		wp_register_script(
			'newsletteredit-'.$block.'-editor',
			$url.$js_path,
			array(
				'wp-blocks',
				'wp-i18n',
				'wp-element',
			),
			filemtime( $dir.$js_path )
		);

		$css_path = 'css/blocks/'.$block.'.css';
		wp_register_style(
			'newsletteredit-'.$block.'-editor',
			$url.$css_path,
			array(
				'wp-blocks',
			),
			filemtime( $dir.$css_path )
		);
		register_block_type( 'eelv-newsletter/'.$block,
			array(
				'editor_script' => 'newsletteredit-'.$block.'-editor',
				'editor_style'  => 'newsletteredit-'.$block.'-editor',
				'render_callback' => 'newsletteredit_'.$block.'_render',
			)
		);
	}
	wp_localize_script('newsletteredit-autocomplete-editor', 'eelv_newsletter_autocomplete', array(
			'editor_custom_variables'=>$eelv_newsletter->custom_variables,
	));
}

add_action('init', 'newsletteredit_block_init');
