<?php
/*
Plugin Name: EELV Newsletter
Plugin URI: https://wordpress.org/plugins/eelv-newsletter/
Description:  Add a registration form on frontOffice, a newsletter manager on BackOffice
Version: 4.8.3
Author: Bastien Ho
Author URI: http://avecnous.eu
License: GPLv2
Text Domain: eelv-newsletter
Domain Path: /languages/
*/
global $eelv_newsletter;
$eelv_newsletter=new EELV_newsletter();

include_once (plugin_dir_path(__FILE__).'blocks/newsletteredit.php');
/**
 * EELV_Newsletter main class
 */
class EELV_newsletter{
    var $pluginversion='4.8';

    /**
     * ID for DB version
     * Beeing updated each time the SQL structure changes
     * @var string
     */
    var $eelv_newsletter_version;

    /**
     * ID for Scripts version
     * Beeing updated each time the static files change
     * @var string
     */
    var $js_version;

    /**
     *
     * @var string
     */
    var $installed_ver;
    /**
     * Current version of the options version
     * Beeing updated each time the configuration page changes
     * @var string
     */
    var $eelv_newsletter_options_version;

    var $newsletter_tb_name;
    var $newsletter_plugin_url;
    var $newsletter_base_url;
    var $lettreinfo_plugin_path;
    var $eelv_nl_content_themes;
    var $eelv_nl_default_themes;
    var $default_item_style;
    var $default_item_style_trads;
    var $newsletter_sql;
    var $mime_type;
    var $custom_variables;

    var $currentTemplate;

    var $news_reg_return;

    public $form_defaults;

    private $widgetid=0;

    var $eol;


    //Initialize the plugin
    function __construct(){
        global $wpdb;

        //Plugin translation
        load_plugin_textdomain( 'eelv-newsletter', false, 'eelv-newsletter/languages' );
        $this->plugin_description=__('Add a registration form on frontOffice, a newsletter manager on BackOffice','eelv-newsletter');
        $this->plugin_name=__('EELV Newsletter','eelv-newsletter');

        $this->news_reg_return=array();
        $this->eelv_newsletter_version = '2.6.5';
        $this->js_version='2.8.0';
        $this->installed_ver = get_option( "eelv_newsletter_version" );
        $this->eelv_newsletter_options_version = 7;
        $this->newsletter_tb_name = 'eelv_'.$wpdb->blogid. '_newsletter_adr';
        $this->newsletter_plugin_url = plugins_url();
        $this->newsletter_base_url = plugins_url('/', __FILE__);
        $this->lettreinfo_plugin_path=plugin_dir_path(__FILE__);
        $this->eelv_nl_content_themes=array();
        $this->eelv_nl_default_themes=array();
        $this->eol=get_option( 'newsletter_eol' )=='n'?"\n":"\r\n";
        $this->newsletter_sql = "CREATE TABLE " . $this->newsletter_tb_name . " (
          `id` mediumint(9) NOT NULL AUTO_INCREMENT,
          `parent` mediumint(9) DEFAULT 0 NOT NULL,
          `nom` VARCHAR(255) DEFAULT '' NOT NULL,
          `email` VARCHAR(255) DEFAULT '' NOT NULL,
          PRIMARY KEY  (`id`)
          );";

        $this->mime_type=get_option( 'newsletter_mime_type' );
        if($this->mime_type==''){
            $this->mime_type='html_txt';
        }

        include_once ($this->lettreinfo_plugin_path.'widget-archives.php');
        include_once ($this->lettreinfo_plugin_path.'widget-subscribe.php');
        include_once ($this->lettreinfo_plugin_path.'reply.php');

        $this->default_item_style=array(
            't_size'=>'thumbnail',
            'div'=>'width:550px; margin:5px 0px;text-align:left; clear:both; border-top:#CCC 1px dotted; padding-top:1em; margin-top:1em;',
            'a'=>'text-decoration:none;color:#666666;',
            'img'=>'float:left; margin-right:10px;',
            'h3' => 'margin:0px !important;text-decoration:none;color:#000000;',
            'p' => '',
            'readmore'=>'',
            'readmore_content'=>'',
            'excerpt_length'=>300,
            'css'=>'',
            'mail.css'=>1,
            'inline.css'=>1,
        );

        $this->default_item_style_trads=array(
            't_size'=>__('Thumbnail size','eelv-newsletter'),
            'div'=>__('Div style:','eelv-newsletter'),
            'a'=>__('A style:','eelv-newsletter'),
            'img'=>__('Img style:','eelv-newsletter'),
            'h3'=>__('H3 style:','eelv-newsletter'),
            'p'=>__('P style:','eelv-newsletter'),
            'readmore'=>__('Readmore style:','eelv-newsletter'),
            'readmore_content'=>__('Readmore content:','eelv-newsletter'),
            'excerpt_length'=>__('Excerpt length: (characters)','eelv-newsletter'),
            'css'=>__('Custom CSS','eelv-newsletter'),
        );

        $this->form_defaults=array(
            'id'=>'',
            'title'      => '',
            'label'      => __('Subscribe our newsletter', 'eelv-newsletter'),
            'input' => __('Newsletter : your email address', 'eelv-newsletter'),
            'button' => __('Ok', 'eelv-newsletter'),
            'options' => '',
            'archives' => '',
            'group'=>1,

            'css_template' => 'default',
            'form_class' => '',
            'input_wrapper_class' => '',
            'label_class' => '',
            'input_class' => '',
            'button_class' => '',

            'form_color'=>'#FFFFFF',
            'form_transparent'=>0,
            'text_color'=>'#333333',
            'text_color_auto'=>0,
            'options_color'=>'#EEEEEE',
            'options_transparent'=>0,
            'options_text_color'=>'#000000',
            'options_color_auto'=>0,
        );

        $this->custom_variables = apply_filters('eelv_newsletter_editor_custom_variables', array(
            'dest_name' => __('Recipient name', 'eelv-newsletter'),
            'dest_login' => __('Login', 'eelv-newsletter'),
            'dest_email'=> __('Email', 'eelv-newsletter'),
            'blog_name'=> __('Blogname', 'eelv-newsletter'),
            'blog_url'=> __('URL', 'eelv-newsletter'),
            'blog_image'=> __('Image', 'eelv-newsletter'),
            'blog_description'=> __('Desc', 'eelv-newsletter'),
        ));

        // Hooks into WordPress
        register_activation_hook(__FILE__,array(&$this,'activate'));
        register_deactivation_hook(__FILE__, array(&$this,'deactivate'));
        add_action('plugins_loaded', array(&$this,'update_db_check'));
        add_action( 'init', array(&$this,'init' ));

        // Cron task
        add_filter( 'cron_schedules', array(&$this,'cron_schedules') );
        add_action( 'eelv_newsletter_cron_tasks', array($this,'autosend') );
        if (wp_next_scheduled('eelv_newsletter_CronTask')) {
            wp_clear_scheduled_hook('eelv_newsletter_CronTask');
        }
        if (wp_next_scheduled('eelv_newsletter_Cron_Task')) {
            wp_clear_scheduled_hook('eelv_newsletter_Cron_Task');
        }
        if (!wp_next_scheduled('eelv_newsletter_cron_tasks')) {
            wp_schedule_event(time(), 'newsly', 'eelv_newsletter_cron_tasks');
        }

        // Admin UI
        add_action('admin_menu', array(&$this,'eelv_news_ajout_menu'));
        add_action( 'network_admin_menu', array(&$this,'eelv_news_ajout_network_menu'));
        add_filter('manage_newsletter_posts_columns', array(&$this,'lettreinfo_columns_head'));
        add_action('manage_newsletter_posts_custom_column', array(&$this,'lettreinfo_columns_content'), 10, 2);
        add_filter('manage_newsletter_archive_posts_columns', array(&$this,'lettreinfo_archives_columns_head'));
        add_action('manage_newsletter_archive_posts_custom_column', array(&$this,'lettreinfo_archives_columns_content'), 10, 2);
        add_filter('manage_newsletter_template_posts_columns', array(&$this,'lettreinfo_template_columns_head'));
        add_action('manage_newsletter_template_posts_custom_column', array(&$this,'lettreinfo_template_columns_content'), 10, 2);

        // Editor
        add_action( 'save_post', array(&$this,'newsletter_save_postdata' ));
        add_action( 'add_meta_boxes', array(&$this,'newsletter_add_custom_box' ));
        add_action('wp_ajax_eelv_newsletter_included_wizard', array(&$this, 'single_included_wizard'));
        add_action('wp_ajax_eelv_newsletter_queue_refresh', array(&$this, 'queue_refresh'));
        add_filter('tiny_mce_before_init', array(&$this,'myformatTinyMCE') );
        add_action('admin_head', array(&$this, 'editor_extend'));

        // Scripts & front
        add_action('wp_head', array(&$this,'inscform_action'));
        add_action('wp_ajax_eelv_newsletter_inscform_action', array(&$this,'inscform_action'));
        add_action('wp_ajax_nopriv_eelv_newsletter_inscform_action', array(&$this,'inscform_action'));
        add_action('admin_head',array(&$this,'add_alert'));
        add_action('wp_enqueue_scripts', array(&$this,'eelv_news_scripts'));
        add_action('admin_enqueue_scripts', array(&$this,'eelv_news_adminscripts'));
        add_action('wp_ajax_eelv_newsletter_custom.css', array(&$this, 'custom_css_raw'));
        add_action('wp_ajax_nopriv_eelv_newsletter_custom.css', array(&$this, 'custom_css_raw'));

        add_filter( 'archive_template', array(&$this,'get_custom_archive_template' ));
        add_filter( 'single_template', array(&$this,'get_custom_single_template' ));

        add_action('admin_post_newsletter_export_address_csv', array( &$this, 'export_address_csv'));

        // Shortcodes / widgets
        add_shortcode( 'eelv_news_form' , array(&$this,'get_news_large_form' ));
        add_shortcode( 'desinsc_url' , array(&$this,'nl_short_desinsc' ));
        add_shortcode( 'nl_date' , array(&$this,'nl_short_date' ));

        add_action('widgets_init', array(&$this,'register_widget'));
    }

    /**
     * PHP4 constructor
     */
    public function EELV_newsletter(){
        $this->__construct();
    }

    /**
     * WP init function
     * @global type $wpdb
     */
    function init(){
        include_once ($this->lettreinfo_plugin_path.'cli.php');

        global $wpdb;
        $cpt_newsletter = include $this->lettreinfo_plugin_path.'inc/cpt/newsletter.php';
        $cpt_newsletter_template = include $this->lettreinfo_plugin_path.'inc/cpt/newsletter_template.php';
        $cpt_newsletter_archive = include $this->lettreinfo_plugin_path.'inc/cpt/newsletter_archive.php';
        $cpt_newsletter_archives_types = include $this->lettreinfo_plugin_path.'inc/cpt/newsletter_archives_types.php';
        register_post_type('newsletter', $cpt_newsletter);
        register_post_type('newsletter_template', $cpt_newsletter_template);
        register_post_type('newsletter_archive', $cpt_newsletter_archive);

        register_taxonomy('newsletter_archives_types',
            array('newsletter','newsletter_archive'),
            $cpt_newsletter_archives_types
        );

        require_once($this->lettreinfo_plugin_path.'templates.php');

        // UPDATE PLUGIN

          if( $this->installed_ver != $this->eelv_newsletter_version ) {
            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
            dbDelta($this->newsletter_sql);
            update_option( 'eelv_newsletter_version', $this->eelv_newsletter_version );
          }
          if(false===$wpdb->query('SELECT `id` FROM '.$this->newsletter_tb_name.' LIMIT 0,1') ){
            $this->activate();
          }

        if(function_exists('wp_register_sidebar_widget')){
            wp_register_sidebar_widget(
             'widget_eelv_lettreinfo_insc',        // your unique widget id
             __('Subscribe newsletter (Deprecated)','eelv-newsletter'),          // widget name
             array(&$this,'widget_eelv_lettreinfo_side'),  // callback function
             array(                  // options
                   'description' => __('This widget is deprecated, please use the new one','eelv-newsletter')
             )
           );
        }
        if(function_exists('wp_register_widget_control')){
            wp_register_widget_control('widget_eelv_lettreinfo_insc', __('Subscribe newsletter (Deprecated)','eelv-newsletter'),array(&$this,'widget_eelv_lettreinfo_insc_control'));
        }
    }

    /* INSTALLATION DES TABLES  */
    function activate() {
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($this->newsletter_sql);
        add_option('eelv_newsletter_version', $this->eelv_newsletter_version);
    }

    // WP 3.1 patch upgrade
    function update_db_check() {
        if (get_option('eelv_newsletter_version') != $this->eelv_newsletter_version) {
          update_option( 'eelv_newsletter_version', $this->eelv_newsletter_version );
          $this->activate();
        }
    }

    function deactivate(){
        wp_clear_scheduled_hook('eelv_newsletter_cron_tasks');
    }

    function editor_extend(){
        if ( ! current_user_can('edit_posts') && ! current_user_can('edit_pages') )
                        return;

        if(get_user_option('rich_editing') == 'false')
            return;

        global $post;
        if(!$post || ($post->post_type!='newsletter' && $post->post_type!='newsletter_template'))
            return;

        add_filter( 'mce_external_plugins', array(&$this,'editor_add_buttons') );
        add_filter( 'mce_buttons_3', array(&$this,'editor_register_buttons') );
    }
    function editor_add_buttons( $plugin_array ) {
        $plugin_array['eelvnewsletter'] = plugins_url( 'js/editor.js' , __FILE__ );
        return $plugin_array;
    }
    function editor_register_buttons( $buttons ) {
        $buttons = array_merge( $buttons, array_keys($this->custom_variables));
        return $buttons;
    }

    /**
     * CSS & JS scripts Admin side
     */
    function eelv_news_adminscripts(){
        $screen = get_current_screen();
        $admin_css_version = (false != $ver = @filemtime(plugin_dir_path(__FILE__).'css/admin.css')) ? $ver : $this->pluginversion;
        $admin_js_version = (false != $ver = @filemtime(plugin_dir_path(__FILE__).'js/admin.js')) ? $ver : $this->pluginversion;
        wp_enqueue_style('eelv_newsletter', plugins_url( 'css/admin.css' , __FILE__ ), null, $admin_css_version);
        wp_enqueue_script('eelv_newsletter_admin', plugins_url( 'js/admin.js' , __FILE__ ), 'jquery', $admin_js_version, true);
        wp_localize_script('eelv_newsletter_admin', 'eelv_newsletter', array(
            'url' => $this->newsletter_base_url,
            // Tracking
            'read_on' => str_replace('\'','\\\'',__('Read on :','eelv-newsletter')),
            'on'=>str_replace('\'','\\\'',__('On :','eelv-newsletter')),
            'from_ip'=>str_replace('\'','\\\'',__('From IP address :','eelv-newsletter')),
            'unread'=>str_replace('\'','\\\'',__('Unread','eelv-newsletter')),
            //Address book
            'move_contacts_to'=>__('Do you really want to move selected contacts to :', 'eelv-newsletter' ),
            'remove_contacts'=>__('Do you really want to remove selected contacts ?', 'eelv-newsletter' ),
            // Editing
            'load_default_content'=>__('Do you want to load this skin\'s default content ?\n\nWarning!\n\nYou will loose the current content.', 'eelv-newsletter' ),
            'screen'=>$screen->base=='post' ? $screen->post_type : 'other',
            'editor_custom_variables'=>$this->custom_variables,
        ));
        if ( 'newsletter_archive' == get_post_type() ){
            wp_dequeue_script( 'autosave' );
        }
    }

    /**
     * CSS & JS scripts Front side
     */
    function eelv_news_scripts() {
        $front_css_version = (false != $ver = @filemtime(plugin_dir_path(__FILE__).'css/newsletter.min.css')) ? $ver : $this->pluginversion;
        $front_js_version = (false != $ver = @filemtime(plugin_dir_path(__FILE__).'js/newsletter.min.js')) ? $ver : $this->pluginversion;
        wp_enqueue_style('eelv_newsletter', plugins_url( 'css/newsletter.min.css' , __FILE__ ), null, $front_css_version);
        wp_enqueue_script('jquery');
        wp_enqueue_script('eelv_newsletter', plugins_url( 'js/newsletter.min.js' , __FILE__ ), 'jquery', $front_js_version,true);
    }


    function prefix_css($css, $prefix){
      return preg_replace('/^(.*)([\{|\,])\s?$/im', $prefix.' $1$2', str_replace('#newsletter_archive', '', $css));
    }



   /*
    * Colums management
    *
    *
    */

    /**
     * ADD NEW COLUMN
     * @param array $defaults
     * @return type
     */
    function lettreinfo_columns_head($defaults) {
        $defaults['envoyer'] = __('Send','eelv-newsletter');
        return $defaults;
    }

    /**
     * COLUMN CONTENT
     * @param type $column_name
     * @param type $post_ID
     */
    function lettreinfo_columns_content($column_name, $post_ID) {
        if ($column_name == 'envoyer') {
            $my_temp=get_post(get_post_meta($post_ID, 'nl_template',true));
            if(0!==get_the_ID() && ''!=get_the_title() && ''!=get_the_content()  && $my_temp){
                echo $my_temp->post_title;
                echo '<br/><a href="edit.php?post_type=newsletter&page=news_envoi&post='.get_the_ID().'">'.__('Preview and send','eelv-newsletter').'</a>';
            }
            else{
                echo __('Not ready yet...','eelv-newsletter');
            }
        }
    }

    /**
     * ADD NEW COLUMN (ARCHIVES)
     * @param type $defaults
     * @return type
     */
    function lettreinfo_archives_columns_head($defaults) {
        $defaults['queue'] = __('Queue','eelv-newsletter');
        $defaults['sent'] = __('Sent','eelv-newsletter');
        $defaults['read'] = __('Readen','eelv-newsletter');
        $defaults['answers'] = __('Answers','eelv-newsletter');
        return $defaults;
    }

    /**
     * COLUMN CONTENT  (ARCHIVES)
     * @param type $column_name
     * @param type $post_ID
     */
    function lettreinfo_archives_columns_content($column_name, $post_ID) {
        if ($column_name == 'queue') {
            $dest = get_post_meta($post_ID, 'destinataires', true);
            echo abs(substr_count($dest, ','));
        }
        if ($column_name == 'sent') {
            $sent = get_post_meta($post_ID, 'sentmails', true);
            echo abs(substr_count($sent, ','));
        }
        if ($column_name == 'read') {
            $spy = get_post_meta($post_ID, 'nl_spy', true);
            if ($spy == 1) {
                $sent = get_post_meta($post_ID, 'sentmails', true);
                $lus = abs(substr_count($sent, ':3'));
                $tot = abs(substr_count($sent, ','));
                if ($tot > 0) {
                    echo $lus . ' (' . round($lus / $tot * 100) . '%)';
                } else {
                    echo '-';
                }
            } else {
                _e('deactivated', 'eelv-newsletter');
            }
        }
        if ($column_name == 'answers') {
            $nl = get_post($post_ID);
            $content = $nl->post_content;
            if (strpos($content, '[nl_reply_link') > -1) {
                $answers = get_post_meta($post_ID, 'eelv_nl_reply');
                echo sizeof($answers);
            } else {
                echo __("No question", 'eelv-newsletter');
            }
        }
    }

    /**
     * ADD NEW COLUMN (TEMPLATES)
     * @param type $defaults
     * @return type
     */
    function lettreinfo_template_columns_head($defaults) {
        $defaults['used_models'] = __('Used', 'eelv-newsletter');
        $defaults['used_sent'] = __('Sent', 'eelv-newsletter');
        return $defaults;
    }

    /**
     * COLUMN CONTENT  (ARCHIVES)
     * @param type $column_name
     * @param type $post_ID
     */
    function lettreinfo_template_columns_content($column_name, $post_ID) {
        if ($column_name == 'used_models') {
            $args = array(
                'post_type' => 'newsletter',
                'meta_key' => 'nl_template',
                'meta_value' => $post_ID
            );
            $query = new WP_Query($args);
            echo $query->found_posts;
        }
        if ($column_name == 'used_sent') {
            $args = array(
                'post_type' => 'newsletter_archive',
                'meta_key' => 'nl_template',
                'meta_value' => $post_ID
            );
            $query = new WP_Query($args);
            echo $query->found_posts;
        }
    }

    /**
     * Adds a box to the main column on the Post and Page edit screens
     */
    function newsletter_add_custom_box() {

        //Native posts & Custom post Newsletter edit interface
        add_meta_box(
                'news-carnet-adresse', __("Edit tools", 'eelv-newsletter'), array(&$this, 'newsletter_admin'), 'newsletter'
        );
        add_meta_box(
                'newsletter_admin_wizard', __("Insert a question", 'eelv-newsletter'), array(&$this, 'newsletter_admin_wizard'), 'newsletter'
        );
        add_meta_box(
                'news-envoi-edit', __("Send", 'eelv-newsletter'), array(&$this, 'newsletter_admin_prev'), 'newsletter', 'side'
        );
        add_meta_box(
                'news-convert-post', __("Send as newsletter", 'eelv-newsletter'), array(&$this, 'custom_box_news_transform'), 'post', 'side'
        );

        //Custom post Newsletter Archive edit interface
        add_meta_box(
                'news-archive_viewer', __("Preview", 'eelv-newsletter'), array(&$this, 'newsletter_archive_admin'), 'newsletter_archive'
        );
        add_meta_box(
                'news-archive_viewerdest', __("Recipients", 'eelv-newsletter'), array(&$this, 'custom_box_dest'), 'newsletter_archive'
        );
        add_meta_box(
                'news-archive_viewerqueue', __("Queue", 'eelv-newsletter'), array(&$this, 'custom_box_queue'), 'newsletter_archive'
        );
        add_meta_box(
                'news-archive_answers', __("Answers", 'eelv-newsletter'), array(&$this, 'custom_box_answers'), 'newsletter_archive'
        );

        //Custom post Newsletter Skin edit interface
        add_meta_box(
                'news-skin_content', __("Default content", 'eelv-newsletter'), array(&$this, 'template_default_content'), 'newsletter_template'
        );
        add_meta_box(
                'news-skin_style', __("Item style", 'eelv-newsletter'), array(&$this, 'template_item_style'), 'newsletter_template'
        );
    }

    /**
     * Add menu and sub-menus
     */
    function eelv_news_ajout_menu() {
      add_submenu_page('edit.php?post_type=newsletter', __('Address book', 'eelv-newsletter' ), __('Address book', 'eelv-newsletter' ), 'publish_posts', 'news_carnet_adresse', array(&$this,'news_carnet_adresse'));
      add_submenu_page('edit.php?post_type=newsletter', __('Send', 'eelv-newsletter' ), __('Send', 'eelv-newsletter' ), 'publish_posts', 'news_envoi', array(&$this,'news_envoi'));
      add_submenu_page('edit.php?post_type=newsletter', __('Configuration/help', 'eelv-newsletter' ), __('Configuration/help', 'eelv-newsletter' ), 'manage_options', 'settings_page', array(&$this,'settings_page'));
      add_submenu_page('edit.php?post_type=newsletter', __('Reload parameters', 'eelv-newsletter' ), __('Reload parameters', 'eelv-newsletter' ), 'manage_options', 'checkdb', array(&$this,'checkdb'));
    }
    //
    /**
     * Network side
     */
    function eelv_news_ajout_network_menu() {
      add_submenu_page('settings.php', __('Newsletter', 'eelv-newsletter' ), __('Newsletter', 'eelv-newsletter' ), 'Super Admin', 'newsletter_network_configuration', array(&$this,'newsletter_network_configuration'));
    }


    /*
     * Usefull functions
     *
     *
     */

    function get_row($id){
      global $wpdb;
      $ret =  $wpdb->get_results("SELECT * FROM `$this->newsletter_tb_name` WHERE `id`='$id'");
      if(is_array($ret) && sizeof($ret)>0){
       return $ret[0];
      }
      return false;
    }

    function update_row($datas, $where){
      global $wpdb;
      $update = $wpdb->update($this->newsletter_tb_name, $datas, $where);
      if(false !== $update){
       return true;
      }
      return false;
    }

    /**
     * Get informations from address book table
     * @global type $wpdb
     * @param type $id
     * @return boolean
     */
    function get_news_meta($id){
      return $this->get_row($id);
    }

    /**
     * Make plain text version from an HTML newsletter
     * @param type $str
     * @return type
     */
    function nl_plain_txt($str){
        $str = preg_replace('/<\s*style.+?<\s*\/\s*style.*?>/si', '', $str);
        $str = str_replace(array('&nbsp;','&laquo;','&raquo;','&rsquo;'),array(' ','"','"',"'"),$str);
          $str = strip_tags($str,'<a>');
        $str=str_replace('</a>','</a>'."\n",$str);
        $str=str_replace('&#038;','&',$str);
        preg_match_all('/<a (.+)?href=[\'"](.*)[\'"][.+]?>(.*)<\/a>/',$str,$links);
        if(is_array($links)){
            foreach($links[0] as $id=>$link){
                $lien  = strip_tags($links[2][$id]);
                if(-1 < $p = strpos($lien,'"')) $lien  = substr($lien,0,$p);
                if(-1 < $p = strpos($lien,'\'')) $lien  = substr($lien,0,$p);
                $lien  = str_replace(' ','%20',$lien);
                $str=str_replace($link,$links[3][$id].' : '.$lien.' ',$str);
            }
        }
        $str=str_replace("\n\n\n","\n\n",$str);
        return $str;
    }

    /**
     * Constructs a multipart body content
     * @param type $str
     * @param type $boundary
     * @return string
     */
    function nl_mime_txt($str,$boundary=''){
        $eol=$this->eol;
        $message = '';
        if($this->mime_type=='html_txt'){
            $message .= '--'.$boundary.$eol;
            $message .= 'Content-Type: text/plain; charset=UTF-8'.$eol;
            $message .= 'Content-Transfer-Encoding: quoted-printable'.$eol.$eol;
            $message .= quoted_printable_encode ($this->nl_plain_txt($str)).$eol.$eol;

            $message .= '--'.$boundary.$eol;
            $message .= 'Content-Type: text/html; charset=UTF-8'.$eol;
            $message .= 'Content-Transfer-Encoding: quoted-printable'.$eol.$eol;
            $message .= quoted_printable_encode ('<html><body>'.$str.'</body></html>').$eol.$eol;
            $message .= '--'.$boundary.'--'.$eol;
        }
        else{
            $message .= ('<html><body>'.$str.'</body></html>').$eol.$eol;
        }
        return $message;
    }

    /**
     *
     * @param boolean $in
     * @return boolean
     */
    function myformatTinyMCE($in)        {
        if(get_post_type()=='newsletter' || get_post_type()=='newsletter_archive' || get_post_type()=='newsletter_template'){
                $in['wpautop']=false;
        }
            return $in;
    }

    /**
     *
     * @param type $post_id
     * @param type $type
     * @return string
     */
    function nl_content($post_id,$type='newsletter'){
        $nl =  get_post($post_id);
        if(is_object($nl)){
            $content=$nl->post_content;
            $content = preg_replace('/<\s*style.+?<\s*\/\s*style.*?>/si', '', $content);
            $content = str_replace(array('&nbsp;','&laquo;','&raquo;','&rsquo;'),array(' ','"','"',"'"),$content);
            $template =  get_post(get_post_meta($post_id,'nl_template',true));
            $this->currentTemplate = $template;
            if($template){
              $content= str_replace('[newsletter]',$content,$template->post_content);
            }
            remove_filter('the_content', 'wpautop');
            $return  = apply_filters('the_content',$content);
            add_filter('the_content', 'wpautop');
            if($return=='' && $content!=''){
                    $return=$content;
            }
            $desinsc_url = get_option( 'newsletter_desinsc_url' );
            $return= str_replace('[nl_date]',date_i18n(get_option('date_format')),$return);
            $return= str_replace('[desinsc_url]',"<a href='".$desinsc_url."' target='_blank' class='nl_a'>".$desinsc_url."</a>",$return);
            return $return;
        }
        return '';
    }

    /**
     *
     * @param type $str
     * @return string
     */
    function localize_string($str=''){
        if(strstr($str,'{blog_name}')){
            $str=str_replace('{blog_name}',get_bloginfo('name'),$str);
        }
        if(strstr($str,'{blog_url}')){
            $str=str_replace('{blog_url}',get_bloginfo('wpurl'),$str);
        }
        if(strstr($str,'{blog_description}')){
            $str=str_replace('{blog_description}',get_bloginfo('description'), $str);
        }
        if(strstr($str,'{blog_image}') && (false !== $header_image = get_header_image())){
            $str=str_replace('{blog_image}', sprintf('<img src="%s" width="%s" alt="%s">', $header_image, '100%', get_bloginfo('name')), $str);
        }
        return $str;
    }

    /**
     *
     * @param string $title
     * @param string $link
     * @return string
     */
    function share_links($title='', $link=''){
        return "<div style='margin:0px;text-align:left; clear:both;font-size:9px; '><span style='display:block;float:left;padding:2px;padding-left:10px;padding-right:10px;background:#888;color:#FFF;'>".__('Share on : ', 'eelv-newsletter' )."</span><a href='http://www.facebook.com/sharer.php?u=".urlencode($link)."&t=".$title."' target='_blank' style='display:block;float:left;padding:2px;padding-left:10px;padding-right:10px;background:#3B5998;color:#FFF;'>Facebook</a><a href='https://twitter.com/intent/tweet?text=".$title."%20".urlencode($link)."' target='_blank' style='display:block;float:left;padding:2px;padding-left:10px;padding-right:10px;background:#2BB7EA;color:#FFF;'>Twitter</a><a href='https://plus.google.com/share?url=".urlencode($link)."' target='_blank' style='display:block;float:left;padding:2px;padding-left:10px;padding-right:10px;background:#DB4B39;color:#FFF;'>Google+</a><a href='http://www.linkedin.com/shareArticle?mini=true&url=".urlencode($link)."&title=".$title."' target='_blank' style='display:block;float:left;padding:2px;padding-left:10px;padding-right:10px;background:#0073B2;color:#FFF;'>Linked in</a></div>&nbsp;\n";
    }



    //Shortcodes
    /**
     *
     * @return type
     */
    function nl_short_date(){
        return date_i18n(get_option('date_format'));
    }

    /**
     *
     * @return type
     */
    function nl_short_desinsc(){
      $desinsc_url = get_option( 'newsletter_desinsc_url' );
          return '<a href="'.$desinsc_url.'" target="_blank" class="nl_a">'.$desinsc_url.'</a>';
    }






    /*
     *
     * Front stuff
     *
     *
     *
     *
     *
     *
     *
     *
     *
     *
     *
     *
     */
    ///////////////////////////////////////////// VALIDATION FORMULAIRE

    /**
     *
     * @global type $wpdb
     */
    function inscform_action(){
        global $wpdb;
        $query='';
        if(isset($_POST['news_email'])){
            $email = stripslashes($_POST['news_email']);
            if (filter_var($email, FILTER_VALIDATE_EMAIL)) {


            $msg = get_option( 'newsletter_msg' ,array(
                'sender'=>'' ,
                'suscribe_title'=>'' ,
                'suscribe'=>'' ,
                'unsuscribe_title'=>'' ,
                'unsuscribe'=>''
                ));
            $sender = $msg['sender'];
            $suscribe_title = $msg['suscribe_title'];
            $suscribe = $msg['suscribe'];
            $unsuscribe_title = $msg['unsuscribe_title'];
            $unsuscribe = $msg['unsuscribe'];
            $suscribegrp = isset($_POST['news_grp']) && is_numeric($_POST['news_grp']) ? $_POST['news_grp'] : 1;

            if($suscribegrp>2){
                $ret =  $wpdb->get_results("SELECT * FROM `$this->newsletter_tb_name` WHERE `id`='".$suscribegrp."' AND `parent`='0'");
                if(!is_array($ret) || sizeof($ret)==0){
                  $suscribegrp=1;
                }
            }
        switch($_POST['news_action']){
          case '1':
            do_action('eelv_newsletter_add_contact_to_addressbook', $email, $suscribegrp);
            $ret =  $wpdb->get_results("SELECT * FROM `$this->newsletter_tb_name` WHERE `email`='".str_replace("'","''",$email)."' AND `parent`='".$suscribegrp."'");
            if(is_array($ret) && sizeof($ret)>0){
              $ret = $ret[0];
              if($ret->parent==2){ // Red list
                $query="UPDATE $this->newsletter_tb_name SET `parent`='".$suscribegrp."' WHERE `email`='".str_replace("'","''",$email)."' AND `parent`='".$suscribegrp."'";
                if($query!='' && false===$wpdb->query($query)){
                  $this->news_reg_return[$_POST['news_id']].=__("An error occured !", 'eelv-newsletter') ;
                }
                elseif($query!=''){
                  $this->news_reg_return[$_POST['news_id']]=__("You have been successfully re-registered", 'eelv-newsletter');
                                  if(!empty($sender) && !empty($suscribe_title) && !empty($suscribe)){
                                          wp_mail($email,$suscribe_title,$suscribe,'From:'.$sender.$this->eol);
                                  }
                }
              }
              else{
                $this->news_reg_return[$_POST['news_id']].=__("Your email is already registered in our mailing-list.", 'eelv-newsletter') ;
              }
            }
            else{
              $query="INSERT INTO $this->newsletter_tb_name (`parent`,`nom`,`email`)
                VALUES (".$suscribegrp.",'".str_replace("'","''",substr($email,0,strpos($email,'@')))."','".str_replace("'","''",$email)."')";
              if($query!='' && false==$wpdb->query($query)){
                $this->news_reg_return[$_POST['news_id']].=__("An error occured !", 'eelv-newsletter');
              }
              elseif($query!=''){
                $this->news_reg_return[$_POST['news_id']].=__("Thank you for your subscription", 'eelv-newsletter');
                                if(!empty($sender) && !empty($suscribe_title) && !empty($suscribe)){
                                          wp_mail($email,$suscribe_title,$suscribe,'From:'.$sender.$this->eol);
                                  }
              }
            }
            break;
          case '0':
            do_action('eelv_newsletter_remove_contact_from_addressbook', $email, $suscribegrp);
            $ret =  $wpdb->get_results("SELECT * FROM `$this->newsletter_tb_name` WHERE `email`='".str_replace("'","''",$email)."'");
            if(is_array($ret) && sizeof($ret)>0){
              $query="UPDATE $this->newsletter_tb_name SET `parent`='2' WHERE `email`='".str_replace("'","''",$email)."'";
              if($query!='' && false===$wpdb->query($query)){
                $this->news_reg_return[$_POST['news_id']].=__("An error occured !", 'eelv-newsletter');
              }
              elseif($query!=''){
                $this->news_reg_return[$_POST['news_id']].=__("Thank you, your email have been deleted from our mailing-list", 'eelv-newsletter');
                                  if(!empty($sender) && !empty($unsuscribe_title) && !empty($unsuscribe)){
                                          wp_mail($email,$unsuscribe_title,$unsuscribe,'From:'.$sender.$this->eol);
                                  }
              }
            }
            else{
              $this->news_reg_return[$_POST['news_id']].=__("Your email does'nt appear in our mailing list. No unsubscribe needed", 'eelv-newsletter');
            }
            break;
         }
      }
      else{
        $this->news_reg_return[$_POST['news_id']].= '<b>'.htmlspecialchars(strip_tags($email)).'</b> : '.__('invalid address', 'eelv-newsletter');
      }
    }
    if (defined('DOING_AJAX') && DOING_AJAX) {
      echo $this->news_reg_return[$_POST['news_id']];
      exit;
    }
  }
  ////////////////////////////////////////////////////////////////////////////////////////////////////// FRONT OFFICE

  function get_widget_id(){
      $this->widgetid++;
      return 'nl-widget-'.$this->widgetid;
  }
  /**
   *
   * @param array $params
   * @return string
   */
    function get_news_form($params){
        $params = wp_parse_args( (array) $params, $this->form_defaults );


        if(empty($params['id'])) $params['id']=$this->get_widget_id();

        $options_class = '';
        $info_class = '';
        if($params['css_template']=='default'){
            $params['form_class'].=' eelv_news_default';
            $params['input_wrapper_class'].=' eelv_news_default';
            $params['input_class'].=' eelv_news_default';
            $params['input_class'].=' eelv_news_default';
            $params['button_class'].=' eelv_news_default';
            $info_class=' eelv_news_default';
            $options_class = 'news_hidden_option';
        }
        elseif($params['css_template']=='bootstrap'){
            $params['form_class'].=' form';
            $params['input_wrapper_class'].=' form-inline';
            $params['input_class'].=' form-control';
            $params['button_class'].=' btn btn-default';
            $info_class=' alert alert-info';
            $options_class = 'help-block';
        }

        $form_color = ($params['form_transparent']!=1 ? 'style="background:'.esc_attr($params['form_color']).';"' : '');
        $text_color = ($params['text_color_auto']!=1 ? 'style="color:'.esc_attr($params['text_color']).';"' : '');
        $options_color = ($params['options_transparent']!=1 ? 'style="background:'.esc_attr($params['options_color']).';"' : '');
        $options_text_color = ($params['options_color_auto']!=1 ? 'style="color:'.esc_attr($params['options_text_color']).';"' : '');

        $form='
        <form action="#newsform'.$params['id'].'" method="post" id="newsform'.$params['id'].'" '
                . 'class="newsform '.esc_attr($params['form_class']).'" '
                . 'onsubmit="if(this.news_email.value==\'\' || this.news_email.value==\''.addslashes($params['input']).'\'){ return false; }" '
                . $form_color. '>'
                . '<input type="hidden" name="news_grp" value="'.abs($params['group']).'" />
                    <input type="hidden" name="news_id" value="'.$params['id'].'" />';
            if(!empty($params['label'])){
                $form.= '<label class="eelv_news_label '.esc_attr($params['label_class']).'" for="news_email'.$params['id'].'" '
                    . $text_color.'">'.$params['label'].'</label>';
            }
            $form.= '<div class="eelv_news_input_wrapper'.$params['input_wrapper_class'].'">'
                . '<input type="email" name="news_email" id="news_email'.$params['id'].'" value=""  placeholder="'.esc_attr($params['input']).'" '
                .  ($params['css_template']=='default' ? $text_color : '')
                . 'class="eelv_news_input '.esc_attr($params['input_class']).'"/>'

                . '<button type="submit" class="eelv_news_button '.esc_attr($params['button_class']).'" '
                . ($params['css_template']=='default' ? $text_color : '').'>'.$params['button'].'</button>'
                . '</div>';

            if($params['options'] || $params['archives']){
                $form.='<div class="'.$options_class.'" '. $options_color. '>';
                if($params['options']==1){
                        $form.='<p>'
                                . '<label for="news_option_1'.$params['id'].'" '
                                . ($params['css_template']=='bootstrap' ? 'class="radio"' : '')
                                . $options_text_color.'">'
                                . '<input type="radio" name="news_action" value="1" id="news_option_1'.$params['id'].'" checked="checked"/>'
                                . '<span class="">'
                                . __("Subscribe", 'eelv-newsletter')
                                . '</span>'
                                . '</label>'
                                . ' '
                                . '<label for="news_option_2'.$params['id'].'" '
                                . ($params['css_template']=='bootstrap' ? 'class="radio"' : '')
                                . $options_text_color.'">'
                                . '<input type="radio" name="news_action" value="0"  id="news_option_2'.$params['id'].'"/> '
                                . __("Unsuscribe", 'eelv-newsletter').'</label>'
                                . '</p>';
                }
                if($params['archives']==1){
                    $form.='<p><a href="'.site_url('/newsletter_archive/').'" '.$options_text_color.'">'.__("Last newsletters", 'eelv-newsletter').'</a></p>';
                }
                $form.='</div>';
            }
            if($params['options']==0){
                $form.='<input type="hidden" name="news_action" value="1">';
            }
            if(isset($this->news_reg_return[$params['id']]) && $this->news_reg_return[$params['id']]!=''){
                $form.='<div class="news_return '.$info_class.'" id="news_return'.$params['id'].'" onclick="document.getElementById(\'news_return'.$params['id'].'\').style.display=\'none\';">'
                .$this->news_reg_return[$params['id']]
                .'</div>';
            }
          $form.='
        </form>';
        return $form;
    }

    /**
     *
     * @global type $wpdb
     * @param array $atts
     * @return string
     */
    function get_news_large_form($atts){
        extract(shortcode_atts(array(
                      'id'=>$this->get_widget_id(),
                      'group'=>1,
                      'subscribe'=>1,
                      'unsubscribe' => 1,
                      'archives' => 1,
                      'archives_title' => __("Last newsletters", 'eelv-newsletter'),
                      'title' => __("Your email:", 'eelv-newsletter'),
                      'placeholder' => __("Your email:", 'eelv-newsletter'),
                      'button_label' => __("ok", 'eelv-newsletter'),
             ), $atts));
        $ret='
          <form action="#'.$id.'" id="'.$id.'" method="post" class="newslform" onsubmit="if(this.news_email.value==\'\' || this.news_email.value==\''.__('Your email', 'eelv-newsletter').'\'){ return false; }">
          <input type="hidden" name="news_grp" value="'.esc_attr($group).'" />
          <input type="hidden" name="news_id" value="'.esc_attr($id).'" />
          <div>
                  <label for="news_l_email_'.$id.'">'
                    . '<span class="news_label">'
                    . $title
                    . '</span> '
                    . '<input type="text" name="news_email" id="news_l_email_'.$id.'" value="'.esc_attr($placeholder).'" placeholder="'.__('Your email', 'eelv-newsletter').'" onfocus="this.select()" />
                  </label>
          </div> ';
        if($subscribe==1){
              $ret.='
            <div>
                  <label for="news_l_option_1_'.$id.'">
                    <input type="radio" name="news_action" value="1" id="news_l_option_1_'.$id.'" checked="checked"/><span class="news_label">&nbsp;'
                      .__("Subscribe", 'eelv-newsletter')
                      .'</span>
                  </label>
            </div>';
        }
        if($unsubscribe==1){
                $ret.='
            <div>
                    <label for="news_l_option_2_'.$id.'">
                      <input type="radio" name="news_action" value="0"  id="news_l_option_2_'.$id.'"/><span class="news_label">&nbsp;'
                        .__('Unsubscribe', 'eelv-newsletter')
                        .'</span>
                    </label>
            </div>';
        }
        if($subscribe==0 && $unsubscribe==0){
            $ret.='<input type="hidden" name="news_action" value="1">';
        }
          $ret.='
        <div><input type="submit" value="'.esc_attr($button_label).'" class="btn"/></div>';

        if(isset($this->news_reg_return[$id]) && $this->news_reg_return[$id] !='' && isset($_POST['news_grp']) && $_POST['news_grp']==$group){
          $ret.='<div class="news_retour">'.$this->news_reg_return[$id].'</div>';
        }
        if($archives==1){
                $ret.='<p><a href="/newsletter_archive/">'.$archives_title.'</a></p>';
        }
        $ret.='
        </form>';
        return apply_filters('newsletter_large_form', $ret, $atts);
    }








        /*
         *
         * Back office stuff
         *
         *
         *
         *
         *
         *
         *
         *
         *
         *
         *
         *
         */

    ////////////////////////////////////////////////////////////////////////////////////////////////////// BACK OFFICE

    /**
     *
     * @global type $wpdb
     * @return type
     */
    function news_liste_groupes(){
      global $wpdb;
      //checkdb();
      $querystr = "SELECT `id`,`nom` FROM `$this->newsletter_tb_name` WHERE `parent`='0' ORDER BY `nom`";
      return $wpdb->get_results($querystr);
    }

    /**
     *
     * @global type $wpdb
     * @param type $groupe
     * @param type $fields
     * @return type
     */
    function news_liste_contacts($groupe,$fields='`id`,`nom`,`email`'){
      global $wpdb;
      if(is_array($groupe)){
        $groupe = implode("' OR `parent`='",$groupe);
      }
      $querystr = "SELECT $fields FROM `$this->newsletter_tb_name` WHERE `parent`='$groupe' GROUP BY `email`ORDER BY `nom`";
      return $wpdb->get_results($querystr);
    }



    /*****************************************************************************************************************************************
      A D D R E S S  B O O K
    ****************************************************************************************************************************************/
    /**
     *
     */
    function export_address_csv(){
        require $this->lettreinfo_plugin_path.'csv.php';
    }
    /**
     *
     * @param string $tmp_import_file
     * @param string $separator
     * @return array
     */
    function parse_csv($tmp_import_file, $separator=';'){
        if (($handle = fopen($tmp_import_file, "r")) !== FALSE) {
            $datas=array();
            while (($data = fgetcsv($handle, 1000, $separator,'"')) !== FALSE) {
                foreach($data as $k=>$v){
                    $data[$k]=$v;
                }
                $datas[]=$data;
            }
            fclose($handle);
        }
        return $datas;
    }
    function news_carnet_adresse(){
        global $wpdb;
        $this->inscform_action();
        //checkdb();

        if(!empty($_POST)){
            if (!wp_verify_nonce(\filter_input(INPUT_POST, 'eelv_newsletter_addressbook', FILTER_SANITIZE_STRING), 'eelv_newsletter_addressbook')) {
                wp_die(__('Security error', 'eelv-newsletter'));
            }
        }
        ?>
        <div class="wrap">
          <div id="icon-edit" class="icon32 icon32-posts-newsletter"><br/></div>
          <h2><?php echo  __("Newsletter", 'eelv-newsletter'); ?></h2>
          <table class="widefat" style="margin-top: 1em;">
            <thead>
              <tr>
                <th scope="col" colspan="2"><?php _e('Address book', 'eelv-newsletter'); ?></th>
              </tr>
            </thead>
            <tbody>
              <tr>
                <td>
        <?php
        $for = 'liste';
        $for2 = 'liste';
        $grp_id = '';
        $con_id = '';
        // Group deletion
        if (isset($_GET['delgroupe'])) {
            $for = 'liste';
            $grp_id = $_GET['delgroupe'];
            $query = "DELETE FROM $this->newsletter_tb_name WHERE `id`='$grp_id' OR `parent`='$grp_id'";
            if (false === $wpdb->query($query)) {
                ?><div class="updated"><p><strong><?php _e('An error occured, no group has been deleted !', 'eelv-newsletter'); ?></strong></p></div><?php
            } else {
                ?><div class="updated"><p><strong><?php _e('Successful deletion !', 'eelv-newsletter'); ?></strong></p></div><?php
            }
            $grp_id = '';
        }
        // Contacts deletion
        if (isset($_GET['delcontacts']) && isset($_GET['liste'])) {
            $grp_id = $_GET['liste'];
            $MBRS = $this->news_liste_contacts($grp_id);
            if (sizeof($MBRS) > 0) {
                $ac = '`id`=0';
                $nb = 0;
                foreach ($MBRS as $contact) {
                    if (isset($_POST['contact_' . $contact->id])) {
                        $ac.=' OR `id`=' . $contact->id;
                        $nb++;
                    }
                }
                $query = "DELETE FROM $this->newsletter_tb_name WHERE $ac";
                if (false === $wpdb->query($query)) {
                    ?><div class="updated"><p><strong><?php _e('An error occured, no contact has been deleted !', 'eelv-newsletter'); ?></strong></p></div><?php
                } else {
                    ?><div class="updated"><p><strong><?php printf(__('%s contacts deleted !', 'eelv-newsletter'), $nb); ?></strong></p></div><?php
                }
            }
        }
        // Contacts move
        if (isset($_GET['ngrp']) && isset($_GET['liste'])) {
            $grp_id = $_GET['liste'];
            $MBRS = $this->news_liste_contacts($grp_id);
            if (sizeof($MBRS) > 0) {
                $ac = '`id`=0';
                $nb = 0;
                foreach ($MBRS as $contact) {
                    if (isset($_POST['contact_' . $contact->id])) {
                        $ac.=' OR `id`=' . $contact->id;
                        $nb++;
                    }
                }
                $query = "UPDATE $this->newsletter_tb_name SET `parent`='" . str_replace("'", "''", $_GET['ngrp']) . "' WHERE $ac";
                if (false === $wpdb->query($query)) {
                    ?><div class="updated"><p><strong><?php _e('An error occured, no contact has been moved !', 'eelv-newsletter'); ?></strong></p></div><?php
                } else {
                    ?><div class="updated"><p><strong><?php printf(__('%s contacts succesfully moved !', 'eelv-newsletter'), $nb); ?></strong></p></div><?php
                }
            }
        }
              if (isset($_GET['groupe'])) {
                  $for = 'groupe';
                  $grp_id = $_GET['groupe'];
              }
              if (isset($_GET['liste'])) {
                  $for = 'liste';
                  $grp_id = $_GET['liste'];
              }
              if (isset($_GET['contact'])) {
                  $for2 = 'contact';
                  $con_id = $_GET['contact'];
              }
              if (isset($_POST['grp_nom'])) {
                  $grp_nom = sanitize_text_field($_POST['grp_nom']);
                  $grp_email = sanitize_text_field($_POST['grp_email']);
                  if (is_numeric($grp_id)) {
                      $query = "UPDATE $this->newsletter_tb_name SET `nom`='" . str_replace("'", "''", $grp_nom) . "', `email`='" . str_replace("'", "''", $grp_email) . "' WHERE `id`='$grp_id'";
                  } else {
                      $query = "INSERT INTO $this->newsletter_tb_name (`nom`,`email`) VALUES ('" . str_replace("'", "''", $grp_nom) . "', '" . str_replace("'", "''", $grp_email) . "')";
                  }
                  if (false === $wpdb->query($query)) {
                      ?><div class="updated"><p><strong><?php _e('An error occured...', 'eelv-newsletter'); ?></strong></p></div><?php
                  } else {
                      $for = 'liste';
                      ?><div class="updated"><p><strong><?php _e('Record saved', 'eelv-newsletter'); ?></strong></p></div><?php
                  }
              }
              if (isset($_POST['con_nom']) && is_numeric($grp_id)) {
                  $con_nom = sanitize_text_field(stripslashes($_POST['con_nom']));
                  $con_email = sanitize_email(stripslashes($_POST['con_email']));
                  $for2 = 'liste';
                  $for = 'liste';
                  if (is_numeric($con_id)) {
                      $query = "UPDATE $this->newsletter_tb_name SET `nom`='" . str_replace("'", "''", $con_nom) . "',`email`='" . str_replace("'", "''", $con_email) . "' WHERE `id`='$con_id'";
                  } else {
                      switch ($_POST['import_type']) {
                          case 'unite':
                              $query = "INSERT INTO " . $this->newsletter_tb_name . " (`parent`,`nom`,`email`)
                              VALUES ('$grp_id','" . str_replace("'", "''", $con_nom) . "','" . str_replace("'", "''", $con_email) . "')";
                              break;
                          case 'masse':
                              $imp = preg_split('/[,;\n\t]/', stripslashes($_POST['con_mul'] . ','));
                              $query = 'INSERT INTO ' . $this->newsletter_tb_name . ' (`parent`,`nom`,`email`) VALUES ';
                              foreach ($imp as $entry) {
                                  $entry = trim($entry);
                                  if (filter_var($entry, FILTER_VALIDATE_EMAIL)) {
                                      $query.="
                                  ('$grp_id','" . str_replace("'", "''", substr($entry, 0, strpos($entry, '@'))) . "','" . str_replace("'", "''", $entry) . "'),";
                                  } elseif ($entry != '') {
                                      echo'<p>' . htmlentities(strip_tags($entry)) . ' : ' . __('invalid address', 'eelv-newsletter') . '</p>';
                                  }
                              }
                              $query = substr($query, 0, -1);
                              $query.="";
                              break;
                          case 'file':
                              $upload_dir = wp_upload_dir();
                              $tmp_import_file = $upload_dir['path'].'/eelv-newsletter-addressbook-import.csv';
                              if(!copy($_FILES['con_file']['tmp_name'], $tmp_import_file)){
                                  ?><div class="updated"><p><strong><?php _e('An error occured while uploading your file...', 'eelv-newsletter'); ?></strong></p></div><?php
                                  $for2='import_error';
                              }
                              $for2='import_file';
                              break;
                          case 'file_':
                              $upload_dir = wp_upload_dir();
                              $tmp_import_file = $upload_dir['path'].'/eelv-newsletter-addressbook-import.csv';
                              $datas = $this->parse_csv($tmp_import_file, $_POST['con_separator']);

                              $schema = array(
                                  'email'=>'',
                                  'name'=>''
                              );

                              foreach ($datas[0] as $col=>$data) {
                                  if(isset($_POST['destination_'.$col])){
                                      $schema[$_POST['destination_'.$col]]=$col;
                                  }
                              }

                              if($schema['email']===''){
                                  ?><div class="updated"><p><strong><?php _e('You did not pick the email column...', 'eelv-newsletter'); ?></strong></p></div><?php
                                  $for2='import_file';
                                  $query='';
                              }
                              else{
                              $query = 'INSERT INTO ' . $this->newsletter_tb_name . ' (`parent`,`nom`,`email`) VALUES ';
                                if(isset($_POST['destination_notfirst'])){
                                    array_shift($datas);
                                }
                                foreach ($datas as $data) {
                                    $email = sanitize_email($data[$schema['email']]);
                                    $name = sanitize_text_field(empty($schema['name']) ? str_replace("'", "''", substr($email, 0, strpos($email, '@'))) : $data[$schema['name']]);
                                    if (filter_var($email, FILTER_VALIDATE_EMAIL)) {
                                        $query.="
                                    ('$grp_id','" . str_replace("'", "''", $name) . "','" . str_replace("'", "''", $email) . "'),";
                                    } elseif ($entry != '') {
                                        echo'<p>' . htmlentities(strip_tags($entry)) . ' : ' . __('invalid address', 'eelv-newsletter') . '</p>';
                                    }
                                }
                                $query = substr($query, 0, -1);
                              }
                              break;
                      }
                  }
                  if ($query != '' && false === $wpdb->query($query)) {
                      ?><div class="updated"><p><strong><?php _e('An error occured !', 'eelv-newsletter'); echo $query; ?></strong></p></div><?php
                  } elseif ($query != '') {
                      ?><div class="updated"><p><strong><?php _e('Record saved', 'eelv-newsletter'); ?></strong></p></div><?php
                  }
              }
              // Group edition
              if ($for == 'groupe') {
                  include($this->lettreinfo_plugin_path.'views/addressbook/group-edit.php');
              }
              if ($for == 'liste') {
                  ////////////////////////////////////////////////////////////Listes
                  if (!is_numeric($grp_id)) { // groupes
                      include($this->lettreinfo_plugin_path.'views/addressbook/group-list.php');
                      ?><p>&nbsp;</p><?php
                  }
                  else { // contacts
                      $news_info = $this->get_news_meta($grp_id);
                      $grp_nom = $news_info->nom;
                      $MBRS = $this->news_liste_contacts($grp_id);
                      $nb_contacts = sizeof($MBRS);
                      ?>
                      <h3 class="sectiontitle title3">
                          <a href='edit.php?post_type=newsletter&page=news_carnet_adresse'>
                              <?php _e('Groups', 'eelv-newsletter'); ?>
                          </a> &raquo;
                          <a href='edit.php?post_type=newsletter&page=news_carnet_adresse&groupe=<?php echo $grp_id; ?>'>
                            <?php echo  $grp_nom ?>
                          </a>
                      </h3>
                      <?php
                      if ($for2 == 'liste') { // liste contact
                          include($this->lettreinfo_plugin_path.'views/addressbook/contact-list.php');
                      }
                      elseif($for2=='import_file' && isset($tmp_import_file)){
                            include($this->lettreinfo_plugin_path.'views/addressbook/contact-import.php');
                      }
                      else { // edit contact
                          include($this->lettreinfo_plugin_path.'views/addressbook/contact-edit.php');
                      }
                  }
              }
              ?>
                      </td></tr></tbody></table>

        <?php if (isset($grp_id) && $grp_id > 0): ?>
                    <p>&nbsp;</p>
                    <table class="widefat">
                        <thead>
                                <tr><th><?php _e('Subscription form for this address book', 'eelv-newsletter'); ?></th></tr>
                        </thead>
                        <tbody>
                                <tr><td>
                                        [eelv_news_form group=<?php echo  $grp_id ?>]
                                </td></tr>
                        </tbody>
                      </table>
        <?php endif; ?>
        </div>
        <?php
    }
  /*****************************************************************************************************************************************
  S E N D I N G
  *****************************************************************************************************************************************/
    function news_envoi(){
            $this->inscform_action();
            global $wpdb;
            $default_exp = get_option( 'newsletter_default_exp' );
            $default_mel = get_option( 'newsletter_default_mel' );
            $desinsc_url = get_option( 'newsletter_desinsc_url' );
            ?>
            <div class="wrap">
                <h2><?php _e('Newsletter: sending', 'eelv-newsletter' ); ?></h2>
              <?php
            if(!isset($_GET['post']) || !is_numeric($_GET['post'])){
                /////////////////////////////////// CHOIX DE LA LETTRE
                $querystr = "SELECT `ID`,`post_title` FROM `$wpdb->posts` WHERE `post_status` = 'publish' AND `post_type` = 'newsletter' ORDER BY `post_name`";
                    $res = $wpdb->get_results($querystr,ARRAY_N);
                    if(sizeof($res)>0){
                wp_enqueue_script('html2canvas',plugins_url( 'js/html2canvas.js' , __FILE__ ));
                      ?><ul class="eelv-newsletter-sending-preview-list"><?php
                      foreach($res as $item){
                  $content=$this->localize_string($this->nl_content( $item[0]));
                        ?>
                <li data-id="<?php echo $item[0]?>">
                    <a href="edit.php?post_type=newsletter&page=news_envoi&post=<?php echo $item[0]?>">
                        <span><?php echo $item[1]?></span>
                    </a>
                    <div class="nl-content-wrapper"><div class="nl-content" id="nl-content-<?php echo $item[0]?>"><?php echo $content; ?></div></div>
                </li>
                <?php
                      }
                      ?></ul><?php
                    }
                    else{
                        _e('No letter is in progress. to create one', 'eelv-newsletter' ); ?> <a href="post-new.php?post_type=newsletter" class="button"><?php _e('click here', 'eelv-newsletter' ); ?></a>
                        <?php
                    }
            }
            else{
                $post_id = $_GET['post'];
                $post = get_post( $post_id );
                if(isset($_GET['convert']) && is_numeric($_GET['convert'])){
                        $content=$post->post_content;
                        if(isset($_GET['add_title'])){
                                $content='<h1>'.$post->post_title.'</h1>'.$content;
                        }
                        if(isset($_GET['add_sharelinks'])){
                                $content.=$this->share_links($post->post_title,$post->guid);
                        }

                        if(0!== $new_post = wp_insert_post( array('post_type'=>'newsletter','post_title' => $post->post_title,  'post_content' => $content,  'post_status' => 'publish'))){
                            add_post_meta($new_post, 'nl_template', $_GET['convert']);
                            $post_id=$new_post;
                            $post = get_post( $new_post);
                            _e("A newsletter has been created", 'eelv-newsletter');
                        }
                        else{
                            _e("Conversion error...", 'eelv-newsletter');
                        }
                }
            if(isset($_GET['settemplate']) && is_numeric($_GET['settemplate'])){
                update_post_meta($post->ID,'nl_template',$_GET['settemplate']);
                }
            $content=$this->localize_string($this->nl_content($post_id));
                //$preview = apply_filters('the_content', $content);

                $reply_url = get_option( 'newsletter_reply_url','');
                if(empty($reply_url)){                                        ?>
                        <div class="updated"><p><a href="edit.php?post_type=newsletter&page=settings_page">
                        <?php _e('Missing parameter "Answer page" for your Newsletter, please go to the configuration page', 'eelv-newsletter' ); ?></a></p></div>
                        <?php
                }
                $desinsc_url = get_option( 'newsletter_desinsc_url' );
                if(empty($desinsc_url)){                                        ?>
                        <div class="updated"><p><a href="edit.php?post_type=newsletter&page=settings_page">
                        <?php _e('Missing parameter "Unsubscribe page" for your Newsletter, please go to the configuration page', 'eelv-newsletter' ); ?></a></p></div>
                        <?php
                }
            $template_id = get_post_meta($post->ID, 'nl_template', true);
                if(!isset($_POST['send']) ){
                include($this->lettreinfo_plugin_path.'views/sending/sending-form.php');
                }
                else{
                /////////////////////////////////// SENDING
                if(0=== $archive = wp_insert_post(
                    array(
                        'post_type'=>'newsletter_archive',
                        'post_title' => esc_attr($_POST['eelv_news_sujet']),
                        'post_content' => $_POST['eelv_news_content'],
                        'post_date' => esc_attr($_POST['post_date']),
                        'post_status' => esc_attr($_POST['eelv_news_stat']),
                        )
                    )
                ){
                    echo __("An error occured !",'eelv-newsletter');
                    }
                    else{
                        $contacts='';
                        // CUSTOM GROUPES
                        $dest = array();
                        $GRPS = $this->news_liste_groupes();
                        foreach($GRPS as $groupe){
                        if(isset($_POST['grp_'.$groupe->id])){
                            array_push($dest,$groupe->id);
                        }
                        }
                        $temp = $this->news_liste_contacts($dest,'email');
                        foreach($temp as $contact){
                        $contacts.=$contact->email.',';
                        }

                    // USERS
                    $result = count_users();
                    foreach($result['avail_roles'] as $role => $count){
                        if(isset($_POST['rol_'.$role])){
                            $blogusers = get_users(array('blog_id'=>$wpdb->blogid,'orderby'=>'nicename','role'=>$role));
                            foreach ($blogusers as $user) {
                                if(in_array($role,$user->roles) || $role==$user->roles){
                                    $contacts.=$user->user_email.',';
                                }
                            }
                        }
                    }

                    // UNITE
                    $temp = preg_split('/[;,\n\t]/',$_POST['dests']);
                    foreach($temp as $contact){
                        if(trim($contact)!=''){
                            $contacts.=trim($contact).',';
                        }
                    }

                    // OTHER
                    $contacts=apply_filters('eelv_newsletter_parse_receipients',$contacts, $archive);
                    $contacts=array_unique(explode(',',$contacts));
                    $contacts=implode(',',$contacts);

                    add_post_meta($archive, 'sujet', $this->localize_string($_POST['eelv_news_sujet']));
                    add_post_meta($archive, 'nl_template', $template_id);
                    add_post_meta($archive, 'expediteur', $_POST['eelv_news_exp']);
                    add_post_meta($archive, 'reponse', $_POST['eelv_news_mel']);
                    add_post_meta($archive, 'destinataires', $contacts);
                    add_post_meta($archive, 'sentmails', '');
                    add_post_meta($archive, 'lastsend', date('Y-m-d H:i:s'));
                    add_post_meta($archive, 'nl_spy', $_POST['eelv_news_spy']);

                    if(isset($_POST['types'])){
                        $types = $_POST['types'];
                        wp_set_object_terms( $archive, $types,'newsletter_archives_types' );
                    }
                    _e('Sending...','eelv-newsletter')."";
                    echo"
                            <script>
                            setTimeout(function(){document.location='post.php?post=".$archive."&action=edit&ref=".time()."';},2000);
                            </script>
                            ".__("To view the delivery status, please go to",'eelv-newsletter')."
                            <a href='edit.php?post_type=newsletter_archive'>".__('archives','eelv-newsletter')."</a>
                            ";
                    $this->autosend();
                    }
            }
            }
            ?>
            </div>
        <?php
    }

    /**
     *
     * @param int $post_id
     * @return void
     */
    function newsletter_save_postdata( $post_id ) {
            if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE )      return;

            //Save temple in NL
            if (isset($_POST['post_type']) && 'newsletter' == $_POST['post_type']  && isset($_REQUEST['newslettertemplate']) && $_REQUEST['newslettertemplate']!=''){
              update_post_meta($post_id, 'nl_template', $_REQUEST['newslettertemplate']);
            }
            //Save default content in template
            if (isset($_POST['post_type']) &&  'newsletter_template' == $_POST['post_type']  && isset($_REQUEST['item_style']) && is_array($_REQUEST['item_style'])) {
              update_post_meta($post_id, 'item_style', $_REQUEST['item_style']);
            }
            if (isset($_POST['post_type']) &&  'newsletter_template' == $_POST['post_type']  && isset($_REQUEST['default_content']) && $_REQUEST['default_content']!='') {
              update_post_meta($post_id, 'default_content', stripslashes($_REQUEST['default_content']));
            }
    }

    /**
     * @return void
     */
    function newsletter_admin_prev() {
        include($this->lettreinfo_plugin_path.'views/admin/editor-send.php');
    }

    /**
     *
     * @global type $wpdb
     */
    function newsletter_admin() {
        include($this->lettreinfo_plugin_path.'views/admin/editor.php');
    }

    /**
     *
     * @param WP_Post $post
     * @param array $item_style
     * @param string $imgsize
     * @param bool $share
     * @param bool $in_editor
     * @return string
     */
    function single_included($post, $item_style, $imgsize, $share=false, $in_editor=false){
        $html=$in_editor?'<div class="eelv-newsletter-single dragable" id="eelv-newsletter-insert-'.$post->ID.'">':'';
        $html.="<div style='".$item_style['div']."'>"
        .(has_post_thumbnail($post->ID) ? "<a href='".get_permalink($post->ID)."' style='".$item_style['a']."'>".get_the_post_thumbnail($post->ID, $imgsize, array('style'=>$item_style['img']))."</a>" : '')
        ."<h3><a href='".get_permalink($post->ID)."' style='".$item_style['h3']."'>".$post->post_title."</a></h3>"
        ."<p style='".$item_style['p']."'>".substr(strip_tags(strip_shortcodes($post->post_content)),0,$item_style['excerpt_length'])."...</p>";
        if($item_style['readmore_content']!=''){
            $html.= "<a href='".get_permalink($post->ID)."' style='".$item_style['readmore']."'>".$item_style['readmore_content']."</a>";
        }
        if($share){
            $html.= $this->share_links($post->post_title, get_post_permalink($post->ID));
        }
        $html.="</div>";
        if($in_editor){
            $html.='</div><!-- /#eelv-newsletter-insert-'.$post->ID.' -->'
                    . '<p class="clear" align="right">'.date_i18n(get_option('date_format') , strtotime($post->post_date)).' '
                    . '<a class="button button-default hide-if-no-js eelv-newsletter-single-add" data-id="eelv-newsletter-insert-'.$post->ID.'">'
                    . '<span class="dashicons dashicons-admin-post"></span> '.__('Insert','eelv-newsletter').'</a>'
                    . '</p>';
        }
        return $html;
    }

    /**
     * @param int $skin
     * @param bool $last
     * @param bool $share
     */
    function single_included_wizard($skin=null, $last=false, $share=false){
        $query=array(
            'status'=>'publish',
            'post_type'=>array('post','page'),
            'posts_per_page'=>5,
            'order_by'=>'date',
            'order'=>'desc'
            );
        if((false !== $q = filter_input(INPUT_POST, 'q', FILTER_SANITIZE_STRING))){
            if(strlen($q)>2){
                $query['s']=$q;
            }
            $share = (filter_input(INPUT_POST, 'share', FILTER_SANITIZE_STRING)=='true');
            $skin = filter_input(INPUT_POST, 'skin', FILTER_SANITIZE_NUMBER_INT);
        }

        $post_list=new WP_Query($query);
        if(count($post_list->posts)){
            $item_style=shortcode_atts($this->default_item_style, get_post_meta($skin, 'item_style',true));
            $imgsize=$item_style['t_size'];
            foreach($post_list->posts as $post){
                echo $this->single_included($post, $item_style, $imgsize, $share, true);
            }
        }
        else{
            echo'<p align="center"><span class="dashicons dashicons-format-quote"></span></p>'
            . '<p align="center">'
            . __('Sorry, there is no posts matching your search...', 'eelv-newsletter')
            . '</p>';

        }
        if($q && filter_input(INPUT_POST, 'action', FILTER_SANITIZE_STRING)=='eelv_newsletter_included_wizard'){
            // Exit after ajax call
            exit;
        }
    }

    /**
     *
     */
    function newsletter_admin_wizard() {
        include($this->lettreinfo_plugin_path.'views/admin/wizard.php');
    }

    /**
     *
     * @global type $wpdb
     */
    function newsletter_archive_admin() {
        include($this->lettreinfo_plugin_path.'views/archive/archive-preview.php');
    }

    /* Meta boxes ******************************************************************************************/

    /**
     *
     */
    function queue_refresh(){
        $target = $_POST['target'];
        if($target=='dest'){
            $this->custom_box_dest();
        }
        if($target=='queue'){
            $this->custom_box_queue();
        }
        exit;
    }

    /**
     *
     * @global type $wpdb
     */
    function custom_box_dest() {
            include($this->lettreinfo_plugin_path.'views/archive/archive-receipients.php');
    }

    /**
     *
     * @param type $post_id
     */
    function progress_queue($post_id){
        include($this->lettreinfo_plugin_path.'views/archive/archive-queue-progress.php');
    }

    /**
     * Queue list
     * @global type $wpdb
     */
    function custom_box_queue() {
            global $wpdb;
            $post_id = isset($_POST['post_id']) ? $_POST['post_id'] : get_the_ID();
        $post=get_post($post_id);
            $send_per_burst = abs(get_option( 'newsletter_send_per_burst_cron',50));
            if($send_per_burst==0){
                $send_per_burst=50;
            }
            $burst_interval = abs(get_option( 'newsletter_burst_interval_cron',5));
            if($burst_interval==0){
                $burst_interval=5;
            }
        $dest = get_post_meta($post_id, 'destinataires',true);
            if(!empty($dest)){
            include($this->lettreinfo_plugin_path.'views/archive/archive-queue-list.php');
        }
        else{
            $this->progress_queue($post_id);
        }
    }

    /**
     * answers
     */
    function custom_box_answers() {
            include($this->lettreinfo_plugin_path.'views/archive/archive-answers.php');
    }

    /**
     *
     * @global type $wpdb
     */
    function custom_box_news_transform(){
            global $wpdb;
            $querystr = "SELECT `ID` FROM $wpdb->posts WHERE post_status = 'publish' AND post_type = 'newsletter_template' ORDER BY `post_title`";
            $IDS = $wpdb->get_col($querystr);
            $templates_nb = sizeof($IDS);
            if($templates_nb>0){ ?>
                <input type="hidden" id="eelv_nl_convert_link" value="edit.php?post_type=newsletter&page=news_envoi&post=<?php echo get_the_ID()?>"/>
                <select name="eelv_nl_convert_id" id="eelv_nl_convert_id">
                  <?php foreach($IDS as $item_id){ ?>
                          <option value="<?php echo $item_id; ?>"><?php echo get_the_title($item_id);?></option>
                  <?php } ?>
                </select>
                <hr/>
                <p><label for="eelv_nl_convert_title"><input type="checkbox" id="eelv_nl_convert_title"  value="1"/> <?php _e("Add title to content",'eelv-newsletter'); ?></label></p>
                <p><label for="eelv_nl_convert_share"><input type="checkbox" id="eelv_nl_convert_share"  value="1"/> <?php _e("Add share links",'eelv-newsletter'); ?></label></p>
                <p><a id="eelv_nl_convert_a" class="button"> <?php _e("Preview and send",'eelv-newsletter'); ?></a></p>
            <?php }
        else{
                _e("No skin available",'eelv-newsletter');
            }
    }

    /**
     *
     */
    function template_default_content(){
            $post_id = get_the_ID();
            $default_content = get_post_meta($post_id, 'default_content',true);
            _e('This content will be loaded when you\'ll choose this template for editing a newsletter.','eelv-newsletter');
            wp_editor( $default_content, 'default_content' );
    }

    /**
     *
     */
    function template_item_style(){
        include($this->lettreinfo_plugin_path.'views/skins/skin-edit.php');
    }

    public function custom_css_raw(){
      header('Content-type: text/css');
      header('Content-disposition: inline; filename: custom.css');
      $template_id = filter_input(INPUT_GET, 'template', FILTER_SANITIZE_NUMBER_INT);
      if(!$template_id){
        echo '/** Missing template ID */';
        exit;
      }
      $skin_item_style = get_post_meta($template_id, 'item_style', true);
      if(!$skin_item_style || !isset($skin_item_style['css'])){
        echo '/** Empty CSS */';
        exit;
      }
      echo '/** Custom CSS */'."\n";
      echo $skin_item_style['css'];
      exit;
    }

    ///////////////////////////////////// CHECK DB
    /**
     *
     * @global type $wpdb
     */
    function checkdb(){
        include($this->lettreinfo_plugin_path.'views/admin/check-db.php');
    }

    /**
     *
     * @global \wpdb $wpdb
     * @param string $courriel
     * @return array
     */
    function getuserinfos($courriel){
            global $wpdb;
            $destin=array(
                    'ID'=>0,
                    'name'=>'',
                    'login'=>''
            );
            $user = $wpdb->get_results($wpdb->prepare(
            "SELECT `ID`,`display_name`,`user_login` FROM ".$wpdb->base_prefix."users WHERE `user_email`='%s'",
            $courriel
            ));

            if($user){
                $user=$user[0];
                $destin['ID']=$user->ID;
                $destin['name']=$user->display_name;
                $destin['login']=$user->user_login;
            }
            else{
                $ret = $wpdb->get_results("SELECT * FROM `$this->newsletter_tb_name` WHERE `email`='".str_replace("'","''",$courriel)."'");
                if(is_array($ret) && sizeof($ret)>0){
                    $destin['name']=$ret[0]->nom;
                    $destin['login']='';
                }
            }
            return $destin;
    }
    ///////////////////////////////////// SEMI CRON AUTO SEND
    /**
     * function cron_schedules
     * used by add_filter( 'cron_schedules')
     *
     * adds custom $schedules to WP_cron
     *
     * 15 minutes
     * @param type $schedules
     * @return type
     */
    function cron_schedules( $schedules ) {
        $burst_interval = abs(get_option( 'newsletter_burst_interval_cron',5));
        if($burst_interval==0){
            $burst_interval=5;
        }
            $schedules['newsly'] = array(
                    'interval' => $burst_interval*60,
                    'display' => __( 'Newsletter burst interval setting', 'eelv-newsletter' )
            );
            return $schedules;
    }

    /**
     *
     * @global type $wpdb
     * @global type $nl_id
     * @global type $dest
     * @param boolean $verbose
     * @filter * @param string $content
     * @param string $email
     * @param eelv_newsletter_content($content, email, $wp_user_datas, $nl_id)
     * @filter the_content($content)
     * @return void
     */
    function autosend($verbose=false, $cli=false, $cli_limit=false){
        global $wpdb,$nl_id,$dest;
        if($cli){
            $verbose=false;
        }
        if(!$cli){
            if(get_option( 'newsletter_send_by')=='cli'){
                if($verbose){
                    echo '<br>'.__('Sending is set on CLI. Please execute <code>wp newsletter send</code>.', 'eelv-newsletter');
                }
                return true;
            }
            $send_per_burst = abs(get_option( 'newsletter_send_per_burst_cron',50));
            if($send_per_burst==0){
                $send_per_burst=50;
            }
            $burst_interval = abs(get_option( 'newsletter_burst_interval_cron',5));
            if($burst_interval==0){
                $burst_interval=5;
            }
            $now = time();
            $last_burst = abs(get_option( 'newsletter_last_burst', 0));
            if($last_burst==0){
                update_option( 'newsletter_last_burst', $now);
                $last_burst = $now;
            }
            $diff = ($now-$last_burst)-($burst_interval*60);

            if($diff<0){
                if($verbose){
                    printf(__('Next try of %s emails in: %s', 'eelv-newsletter'), $send_per_burst, human_time_diff( time()-$diff ));
                    echo' <a href="edit.php?post_type=newsletter&page=settings_page">'.__('Manage burst size and frequency in the configuration page', 'eelv-newsletter').'</a>';
                }
                return true;
            }
        }
        update_option( 'newsletter_last_burst', time());

        $querystr = "SELECT $wpdb->posts.`ID` "
                . "FROM $wpdb->posts,$wpdb->postmeta "
                . "WHERE (post_status = 'publish' OR post_status = 'private') "
                    . "AND post_type = 'newsletter_archive'  "
                    . "AND $wpdb->postmeta.`post_id`=$wpdb->posts.`ID` "
                    . "AND $wpdb->postmeta.`meta_key`='destinataires' "
                    . "AND $wpdb->postmeta.`meta_value`!='' "
                    . "AND post_date<='".current_time('mysql', 0)."'";
        $IDS = $wpdb->get_col($querystr);
        $send_nb = sizeof($IDS);
        if($verbose){
            echo '<br>'.__('Test burst...', 'eelv-newsletter');
        }
        if(!$send_nb){
            $message = __('No burst needed', 'eelv-newsletter');
            if($cli){
                WP_CLI::line($message);
            }
            elseif($verbose){
                echo '<br>'.$message;
            }
            return true;
        }
        if($verbose){
            echo '<br>'.__('Send burst !', 'eelv-newsletter');
        }
        $env=0;
        remove_filter('the_content', 'wpautop');

        foreach($IDS as $nl_id){
            $transient_key='eelv_newsletter_lock-'.$nl_id;
            if(false != $locked_until = get_transient($transient_key)){
                if($verbose){
                    echo'<p>'.sprintf(__('Newsletter #%1$s is locked until %2$s!', 'eelv-newsletter'), $nl_id, date_i18n(get_option('date_format').' '.get_option('time_format'), $locked_until)).'</p>';
                }
                continue;
            }
            // Lock Newsletter
            $locked_for = $burst_interval*MINUTE_IN_SECONDS;
            set_transient($transient_key, strtotime('+'.$burst_interval.' minutes'), $locked_for);

            $my_temp=get_post_meta($nl_id, 'nl_template',true);
            $sujet = mb_encode_mimeheader(get_post_meta($nl_id, 'sujet', true), 'UTF-8');
            $expediteur = mb_encode_mimeheader(get_post_meta($nl_id, 'expediteur', true), 'UTF-8');
            $reponse = mb_encode_mimeheader(get_post_meta($nl_id, 'reponse', true), 'UTF-8');
            $dests_str = get_post_meta($nl_id, 'destinataires',true);
            $nl_spy = get_post_meta($nl_id, 'nl_spy',true);
            if(substr($dests_str,0,1)==','){
                $dests_str=substr($dests_str,1);
            }
            $dests = explode(',', $dests_str);
            $nb_dests = count($dests);
            $allready_sent = get_post_meta($nl_id, 'sentmails',true);
            $template=get_post($my_temp);
            $nl=get_post($nl_id);
            if(!$template){
                delete_transient($transient_key);
                $message = __('Template error... abording', 'eelv-newsletter');
                if($cli){
                    WP_CLI::warning($message);
                }
                elseif($verbose){
                    echo '<br>'.$message;
                }
                continue;
            }
            $skin_item_style=shortcode_atts($this->default_item_style, get_post_meta($template->ID, 'item_style', true));
            if(isset($skin_item_style['mail.css']) && $skin_item_style['mail.css']){
              $content_top='<link rel="stylesheet" href="'.plugins_url( 'css/mail.css' , __FILE__ ).'" type="text/css" media="all" />';
              $content_top.='<style type="text/css">';
              $content_top.=file_get_contents(plugin_dir_path(__FILE__ ).'css/mail.css');
              $content_top.='</style>';
            }
            if(isset($skin_item_style['css']) && $skin_item_style['css']){
              $content_top='<link rel="stylesheet" href="'.add_query_arg(array('action'=>'eelv_newsletter_custom.css', 'template'=>$template->ID), admin_url( 'admin-ajax.php')).'" type="text/css" media="all" />';
              $content_top.='<style>'.$skin_item_style['css'].'</style>';
            }

            $content_top.='<center><a href="'.home_url().'/?post_type=newsletter_archive&p='.$nl_id.'" target="_blank"><font size="1">'.__('Click here if you cannot read this e-mail','eelv-newsletter').'</font></a></center>';
            $content=$nl->post_content;
            $the_content=$content;

            $prov = getenv("SERVER_NAME");
            $boundary=md5(time());
            $headers  = "From: \"$expediteur\" <$reponse>".$this->eol;
            $headers .= "Reply-To: \"$expediteur\" <$reponse>".$this->eol;
            $headers .= "Return-Path: \"$expediteur\" <$reponse>".$this->eol;
            $headers .= "Return-Path: \"$expediteur\" <$reponse>".$this->eol;
            $headers .= "Message-ID: <".$nl_id.time()."@".$prov.">".$this->eol;
            $headers .= "X-Mailer: EELV-Newsletter ".$this->pluginversion.$this->eol;
            if($this->mime_type=='html_txt'){
                $headers .= "Content-Type: multipart/alternative; boundary=".$boundary.$this->eol;
            }
            else{
                $headers .= "Content-Type: text/html; charset=UTF-8".$this->eol;
                $headers .= 'Content-Transfer-Encoding: 8bit'.$this->eol;
            }
            $headers .= 'MIME-Version: 1.0';

            //print_r($dests);
            $this->newsletter_admin_surveillance = get_site_option( 'newsletter_admin_surveillance' );
            if($this->newsletter_admin_surveillance!=''){
              mail($this->newsletter_admin_surveillance,'[EELV-newsletter:'.__('Sending','eelv-newsletter').'] '.$sujet,$this->nl_mime_txt($content,$boundary),$headers,'-f '.$reponse);
            }
            $sent_this_time='';
            $message = sprintf(__('Sending newsletter #%d to %d receipients', 'eelv-newsletter'), $nl_id, $cli_limit ? min(array($cli_limit, $nb_dests)) : $nb_dests);
            if($cli){
                WP_CLI::log($message);
                $progress = \WP_CLI\Utils\make_progress_bar( __('Sending progress', 'eelv-newsletter'), $cli_limit ? min(array($cli_limit, $nb_dests)) : $nb_dests );
                $stats = array('ok'=>0, 'ko'=>0, 'bl'=>0, 'er'=>0);
            }
            elseif($verbose){
                echo '<br>'.$message;
            }
            while($destinator = array_shift($dests)){
                $dest=str_replace(' ', '', trim($destinator));
                if($verbose){
                    echo ', '.$dest;
                }
                if(strstr($the_content,'{')){
                    $destinataire=$this->getuserinfos($dest);
                }
                else{
                    $destinataire=true;
                }
                if ($destinataire && filter_var($dest, FILTER_VALIDATE_EMAIL)) {
                    $ret = $wpdb->get_results("SELECT * FROM `$this->newsletter_tb_name` WHERE `email`='".str_replace("'","''",$dest)."' AND `parent`='2' LIMIT 0,1");
                    if(is_array($ret) && sizeof($ret)==0){             // White liste OK
                        $the_content=apply_filters('eelv_newsletter_content', $content, $dest, $destinataire, $nl_id);
                        $the_sujet=apply_filters('eelv_newsletter_content', $sujet, $dest, $destinataire, $nl_id);
                        if(strstr($the_sujet,'{dest_name}')){
                            $the_sujet=str_replace('{dest_name}',$destinataire['name'],$the_sujet);
                        }
                        if(strstr($the_sujet,'{dest_login}')){
                            $the_sujet=str_replace('{dest_login}',$destinataire['login'],$the_sujet);
                        }
                        if(strstr($the_sujet,'{dest_email}')){
                            $the_sujet=str_replace('{dest_email}',$dest,$the_sujet);
                        }


                        if(strstr($the_content,'{dest_name}')){
                            $the_content=str_replace('{dest_name}',$destinataire['name'],$the_content);
                        }
                        if(strstr($the_content,'{dest_login}')){
                            $the_content=str_replace('{dest_login}',$destinataire['login'],$the_content);
                        }
                        if(strstr($the_content,'{dest_email}')){
                            $the_content=str_replace('{dest_email}',$dest,$the_content);
                        }

                        $the_content=$content_top.apply_filters('the_content',$the_content);

                        if($nl_spy==1){
                            $the_content.='<a href="'.get_bloginfo('url').'"><img src="'.$this->newsletter_base_url.'reading/'.base64_encode($dest.'!'.$nl_id).'/logo.png" border="none" alt="'.get_bloginfo('url').'"/></a>';
                        }


                        if(mail($dest,$the_sujet,$this->nl_mime_txt($the_content,$boundary),$headers,'-f '.$reponse)){  // Envoi OK
                            $sent = '1';
                            $env++;
                            $stats['ok']++;
                        }
                        else{                    // Envoi KO
                            $sent = '0';
                            $stats['ko']++;
                        }
                    }
                    elseif(is_array($ret) && sizeof($ret)>0){           // Black list
                        $sent = '2';
                        $stats['bl']++;
                    }
                    else{                         // Envoi OK
                        $sent = '0+';
                        $stats['ok']++;
                    }
                }
                else{            // Mail invalide
                    $sent = '-1';
                    $stats['er']++;
                }
                $sent_this_time = $dest.':'.$sent.','.$sent_this_time;
                if(!$cli && $env>=$send_per_burst){
                    if($verbose){
                        echo '<br>'.$env.', '.__('done for this burst !', 'eelv-newsletter');
                    }
                    break;
                }
                if($cli){
                    $progress->tick();
                    if($cli_limit && $env>=$cli_limit){
                        break;
                    }
                }
            }//endwhile
            update_post_meta($nl_id, 'destinataires', implode(',',$dests));
            update_post_meta($nl_id, 'sentmails', $sent_this_time.$allready_sent);
            // Unlock Newsletter
            delete_transient($transient_key);
            $message = __('Receipients removed from queue and stored in database.', 'eelv-newsletter');
            if($cli){
                $progress->finish();
                WP_CLI::line($message);
                WP_CLI::log('Statistics:');
                WP_CLI::log("- Sent: ".$stats['ok']);
                WP_CLI::log("- Not sent: ".$stats['ko']);
                WP_CLI::log("- Black-listed: ".$stats['bl']);
                WP_CLI::log("- Bad addresses: ".$stats['er']);
            }
            elseif($verbose){
                echo'<p>'.$message.'</p>';
            }
            if(!$cli && $env>=$send_per_burst){
                break;
            }
            if($cli && $cli_limit && $env>=$cli_limit){
                break;
            }
        }//endforeach
        return true;
    }

    /*****************************************************************************************************************************************
                    C O N F I G U R A T I O N
    *****************************************************************************************************************************************/

    /**
     *
     */
    function newsletter_network_configuration(){
      if( filter_input(INPUT_POST, 'type')=='update') {
            update_site_option( 'newsletter_admin_surveillance', filter_input(INPUT_POST, 'newsletter_admin_surveillance'));
        wp_safe_redirect(admin_url('/network/settings.php?page=newsletter_network_configuration&updated=yes'));
        exit;
      }
      if( filter_input(INPUT_GET, 'updated')) {
            ?>
        <div class="updated"><p><strong><?php _e('Options saved', 'eelv-newsletter' ); ?></strong></p></div>
        <?php
      }
      $this->newsletter_admin_surveillance = get_site_option( 'newsletter_admin_surveillance' );
      include($this->lettreinfo_plugin_path.'views/admin/network-settings.php');
    }

    /**
     *
     */
    function add_alert() {
            if ( current_user_can('manage_options') && get_option( 'newsletter_options_version')!=$this->eelv_newsletter_options_version && !is_network_admin()) {
        ?>
            <div class="updated"><p><a href="edit.php?post_type=newsletter&page=settings_page">
                    <?php _e('New options are available for your Newsletter, please go to the configuration page', 'eelv-newsletter' ); ?></a></p></div>
            <?php
            }
    }

    /**
     * mt_toplevel_page() displays the page content for the custom Test Toplevel menu
     * @TODO : store options in a serialized variable
     * @global type $wpdb
     */
    function settings_page() {
            global $wpdb;

            if(\filter_input(INPUT_POST, 'type', FILTER_SANITIZE_STRING) == 'update' ) {
                if (!wp_verify_nonce(\filter_input(INPUT_POST, 'newsletter_update_options', FILTER_SANITIZE_STRING), 'newsletter_update_options')) {
                wp_die(__('Security error', 'eelv-newsletter'));
                }
                update_option( 'newsletter_default_exp', \filter_input(INPUT_POST, 'newsletter_default_exp', FILTER_SANITIZE_STRING));
                update_option( 'newsletter_default_mel', \filter_input(INPUT_POST, 'newsletter_default_mel', FILTER_SANITIZE_EMAIL));
                update_option( 'newsletter_desinsc_url', \filter_input(INPUT_POST, 'newsletter_desinsc_url', FILTER_SANITIZE_URL));
                update_option( 'newsletter_reply_url', \filter_input(INPUT_POST, 'newsletter_reply_url', FILTER_SANITIZE_URL));
                update_option( 'newsletter_precoch_rs', \filter_input(INPUT_POST, 'newsletter_precoch_rs', FILTER_SANITIZE_NUMBER_INT));
                update_option( 'newsletter_spy_text',\filter_input(INPUT_POST, 'newsletter_spy_text', FILTER_SANITIZE_STRING));
                update_option( 'newsletter_mime_type', \filter_input(INPUT_POST, 'newsletter_mime_type', FILTER_SANITIZE_STRING));
                update_option( 'newsletter_eol', \filter_input(INPUT_POST, 'newsletter_eol', FILTER_SANITIZE_STRING));
                update_option( 'newsletter_send_per_burst_cron', \filter_input(INPUT_POST, 'newsletter_send_per_burst_cron', FILTER_SANITIZE_NUMBER_INT));
                update_option( 'newsletter_burst_interval_cron', \filter_input(INPUT_POST, 'newsletter_burst_interval_cron', FILTER_SANITIZE_NUMBER_INT));
                update_option( 'newsletter_send_by', \filter_input(INPUT_POST, 'newsletter_send_by', FILTER_SANITIZE_STRING));

                update_option( 'newsletter_msg', array(
                        'sender'=>\filter_input(INPUT_POST, 'newsletter_msg_sender', FILTER_SANITIZE_EMAIL),
                        'suscribe_title'=>\filter_input(INPUT_POST, 'newsletter_msg_suscribe_title', FILTER_SANITIZE_STRING),
                        'suscribe'=>\filter_input(INPUT_POST, 'newsletter_msg_suscribe', FILTER_SANITIZE_STRING),
                        'unsuscribe_title'=>\filter_input(INPUT_POST, 'newsletter_msg_unsuscribe_title', FILTER_SANITIZE_STRING),
                        'unsuscribe'=>\filter_input(INPUT_POST, 'newsletter_msg_unsuscribe', FILTER_SANITIZE_STRING)
                ));
                //update_option( 'affichage_NL_hp', $_REQUEST['affichage_NL_hp'] );
                ?>
                <div class="updated"><p><strong><?php _e('Options saved', 'eelv-newsletter' ); ?></strong></p></div>
                <?php
            }
            update_option( 'newsletter_options_version', $this->eelv_newsletter_options_version );
            $default_exp = get_option( 'newsletter_default_exp' );
            $default_mel = get_option( 'newsletter_default_mel' );
            $desinsc_url = get_option( 'newsletter_desinsc_url' );
            $reply_url = get_option( 'newsletter_reply_url' );
            $precoch_rs = get_option( 'newsletter_precoch_rs' );
            $spy_text = get_option( 'newsletter_spy_text' ,str_replace(array('http://','https://'),'',get_bloginfo('url')));
            $mime_type = get_option( 'newsletter_mime_type' );
            $send_by = get_option( 'newsletter_send_by', 'ajax' );
            $eol = get_option( 'newsletter_eol' );
            $send_per_burst = abs(get_option( 'newsletter_send_per_burst_cron',50));
            if($send_per_burst==0){
                $send_per_burst=50;
            }
            $burst_interval = abs(get_option( 'newsletter_burst_interval_cron',5));
            if($burst_interval==0){
                $burst_interval=5;
            }

            if($spy_text==''){
                $spy_text=str_replace(array('http://','https://'),'',get_bloginfo('url'));
            }
            //$affichage_NL_hp = get_option( 'affichage_NL_hp' );

            $this->newsletter_msg = get_option( 'newsletter_msg' );
            $msg_sender = $this->newsletter_msg['sender'];
            $msg_suscribe_title = $this->newsletter_msg['suscribe_title'];
            $msg_suscribe = $this->newsletter_msg['suscribe'];
            $msg_unsuscribe_title = $this->newsletter_msg['unsuscribe_title'];
            $msg_unsuscribe = $this->newsletter_msg['unsuscribe'];
        include($this->lettreinfo_plugin_path.'views/admin/admin-settings.php');
        $this->checkdb();
    }

    /**
     *
     * @param type $template_names
     * @param type $load
     * @param type $require_once
     * @return string
     */
    function locate_plugin_template($template_names, $load = false, $require_once = true ){
        if ( !is_array($template_names) )
            return '';

        $located = '';
        $this_plugin_dir = WP_PLUGIN_DIR.'/'.str_replace( basename( __FILE__), "", plugin_basename(__FILE__) );
        foreach ( $template_names as $template_name ) {
            if ( !$template_name )
              continue;
            if ( file_exists(STYLESHEETPATH . '/' . $template_name)) {
              $located = STYLESHEETPATH . '/' . $template_name;
              break;
            } else if ( file_exists(TEMPLATEPATH . '/' . $template_name) ) {
              $located = TEMPLATEPATH . '/' . $template_name;
              break;
            } else if ( file_exists( $this_plugin_dir .  $template_name) ) {
              $located =  $this_plugin_dir . $template_name;
              break;
            }
        }
        if ( $load && '' != $located )
            load_template( $located, $require_once );
        return $located;
    }

    /**
     *
     * @global type $wp_query
     * @param type $template
     * @return type
     */
    function get_custom_archive_template($template){
      global $wp_query;
      if($wp_query->get_queried_object()->query_var=='newsletter_archive') {
              $templates = array('archive-newsletter_archive.php', 'archive.php');
              $template = $this->locate_plugin_template($templates);
      }
      return $template;
    }

    /**
     *
     * @global type $wp_query
     * @param type $template
     * @return type
     */
    function get_custom_single_template($template){
      global $wp_query;
      $object = $wp_query->get_queried_object();
      if ( is_object($object) && 'newsletter_archive' == $object->post_type ) {
            $templates = array('single-' . $object->post_type . '.php', 'single.php');
            $template = $this->locate_plugin_template($templates);
      }
      return $template;
    }

    ////////////////////////////////////////////////////////////////////////////////////////////////////// WIDGET

    /**
     *
     */
    function register_widget(){
            if(class_exists('EELV_NL_Archives_Widget')){
                    register_widget('EELV_NL_Archives_Widget');
            }
        if(class_exists('EELV_NL_Subscribe_Widget')){
                    register_widget('EELV_NL_Subscribe_Widget');
            }
    }

    /**
     *
     * @param type $params
     */
    function widget_eelv_lettreinfo_side($params) {
            $this->eelv_li_xs_title= get_option('eelv_li_xs_title');
            echo $params['before_widget'];
            echo $params['before_title'];
            echo  $this->eelv_li_xs_title;
            echo $params['after_title'];
            // Call new function with old argument
            echo apply_filters('eelv_newsletter_widget_form',
            $this->get_news_form(
                array(
                    'id'=>'widget',
                    'archives' => get_option('eelv_li_xs_archives',0),
                    'title' => get_option('eelv_li_xs_title'),
                    'options' => get_option('eelv_li_xs_options',0),
                    'form_class' => get_option('eelv_li_xs_formclass',''),
                    'text_class' => get_option('eelv_li_xs_textclass',''),
                    'button_class' => get_option('eelv_li_xs_buttonclass',''),

                    'button' => get_option('eelv_li_xs_buttontext','ok'),
                    'label' => get_option('eelv_li_xs_labeltext',__('Subscribe our newsletter', 'eelv-newsletter')),
                    'input' => get_option('eelv_li_xs_texttext',__('Newsletter : your email address', 'eelv-newsletter')),
                )
            )
        );
       echo $params['after_widget'];
    }

    /**
     *
     */
    function widget_eelv_lettreinfo_insc_control(){
       echo'<p>'.__('This widget is deprecated, please use the new one', 'eelv-newsletter').'</p>';
    }
}
//End of the EELV_newsletter Class
